@@ -17,11 +17,11 @@ Test::Mock::Redis - use in place of Redis for unit testing
 
 =head1 VERSION
 
-Version 0.18
+Version 0.20
 
 =cut
 
-our $VERSION = '0.18';
+our $VERSION = '0.20';
 
 =head1 SYNOPSIS
 
@@ -164,9 +164,41 @@ sub shutdown {
 }
 
 sub set {
-    my ( $self, $key, $value ) = @_;
+    my ( $self, $key, $value, @args ) = @_;
+    my $expires = 0;
+    while (my $option = shift @args) {
+        $option = lc $option;
+        # Only set if key exists
+        if ($option eq 'xx') {
+            return unless $self->exists($key);
 
+        # Only set if key doesn't exist
+        } elsif ($option eq 'nx') {
+            return if $self->exists($key);
+
+        # Set expire time (in seconds)
+        } elsif ($option eq 'ex') {
+            my $new_expires = shift @args;
+            if ($new_expires > $expires) {
+                $expires = $new_expires;
+            }
+
+        # Set expire time (in milliseconds)
+        } elsif ($option eq 'px') {
+            my $new_expires = shift @args;
+            $new_expires /= 1000; # To seconds
+            if ($new_expires > $expires) {
+                $expires = $new_expires;
+            }
+        } else {
+            confess '[error] ERR syntax error';
+        }
+    }
     $self->_stash->{$key} = "$value";
+    if ($expires) {
+        $self->expire($key, $expires);
+    }
+
     return 'OK';
 }
 
@@ -463,26 +495,34 @@ sub llen {
 sub lrange {
     my ( $self, $key, $start, $end ) = @_;
 
-    return @{ $self->_stash->{$key} }[$start..$end];
+    my $array = $self->_stash->{$key};
+    ($start,$end) = _normalize_range(scalar(@$array),$start,$end);
+    return @{ $array }[$start..$end];
 }
 
 sub ltrim {
     my ( $self, $key, $start, $end ) = @_;
 
-    $self->_stash->{$key} = [ @{ $self->_stash->{$key} }[$start..$end] ];
+    my $array = $self->_stash->{$key};
+    ($start,$end) = _normalize_range(scalar(@$array),$start,$end);
+    $self->_stash->{$key} = [ @{ $array }[$start..$end] ];
     return 'OK';
 }
 
 sub lindex {
     my ( $self, $key, $index ) = @_;
 
-    return $self->_stash->{$key}->[$index];
+    my $array = $self->_stash->{$key};
+    $index = _normalize_index(scalar(@$array),$index);
+    return $array->[$index];
 }
 
 sub lset {
     my ( $self, $key, $index, $value ) = @_;
 
-    $self->_stash->{$key}->[$index] = "$value";
+    my $array = $self->_stash->{$key};
+    $index = _normalize_index(scalar(@$array),$index);
+    $array->[$index] = "$value";
     return 'OK';
 }
 
@@ -737,7 +777,7 @@ sub hexists {
     confess '[hexists] ERR Operation against a key holding the wrong kind of value'
          if $self->exists($key) and !$self->_is_hash($key);
 
-    return exists $self->_stash->{$key}->{$hkey};
+    return exists $self->_stash->{$key}->{$hkey} ? 1 : 0;
 }
 
 sub hdel {
@@ -995,7 +1035,8 @@ sub zrevrank {
 sub zrange {
     my ( $self, $key, $start, $stop, $withscores ) = @_;
 
-    $stop = $self->zcard($key)-1 if $stop >= $self->zcard($key);
+    my $length = $self->zcard($key);
+    ($start,$stop) = _normalize_range($length,$start,$stop);
 
     return map { $withscores ? ( $_, $self->zscore($key, $_) ) : $_ }
                ( map { $_->[0] }
@@ -1009,11 +1050,12 @@ sub zrange {
 sub zrevrange {
     my ( $self, $key, $start, $stop, $withscores ) = @_;
 
-    $stop = $self->zcard($key)-1 if $stop >= $self->zcard($key);
+    my $length = $self->zcard($key);
+    ($start,$stop) = _normalize_range($length,$start,$stop);
 
     return map { $withscores ? ( $_, $self->zscore($key, $_) ) : $_ }
                ( map { $_->[0] }
-                     sort { $b->[1] <=> $a->[1] }
+                     sort { $b->[1] <=> $a->[1] || $b->[0] cmp $a->[0] }
                          map { [ $_, $self->_stash->{$key}->{$_} ] }
                              keys %{ $self->_stash->{$key} }
                )[$start..$stop]
@@ -1164,8 +1206,12 @@ The following people have contributed to I<Test::Mock::Redis>:
 
 =over
 
+=item * Chris Reinhardt
+
 =item * Ian Burrell
 
+=item * Gianni Ceccarelli
+
 =item * Karen Etheridge
 
 =item * Keith Broughton
@@ -1174,11 +1220,15 @@ The following people have contributed to I<Test::Mock::Redis>:
 
 =item * Neil Bowers
 
+=item * Nigel Gregoire
+
+=item * Yaakov Shaul
+
 =back
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011, 2012, 2013 Jeff Lavallee.
+Copyright 2015 Jeff Lavallee.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
@@ -1189,6 +1239,22 @@ See L<http://dev.perl.org/licenses/> for more information.
 
 =cut
 
+sub _normalize_index {
+    my ( $length, $index ) = @_;
+
+    $index += $length if $index < 0;
+    return $index;
+}
+
+sub _normalize_range {
+    my ( $length, $start, $end ) = @_;
+
+    $start = _normalize_index($length,$start);
+    $end = _normalize_index($length,$end);
+    $end = $length-1 if $end >= $length;
+
+    return ($start,$end);
+}
 
 sub _is_list {
     my ( $self, $key ) = @_;
